==> skktools_1.0.orig.tar.gz: skktools-1.0.orig/skkdic-sort.c <==
/* SKK JISYO TOOLS (SKK dictionary handling tools)
Copyright (C) 1994, 1996, 1999, 2000
      Hironobu Takahashi, Masahiko Sato, Kiyotaka Sakai

Author: Hironobu Takahashi, Masahiko Sato, Kiyotaka Sakai, Kenji Yabuuchi
Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
Version: $Id: skkdic-sort.c,v 1.3 2000/10/05 17:16:44 czkmt Exp $
Keywords: japanese
Last Modified: $Date: 2000/10/05 17:16:44 $

This file is part of Daredevil SKK.

SKK JISYO TOOLS are free software; you can redistribute them and/or modify
them under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

SKK JISYO TOOLS are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SKK; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* skkdic-sort.c
   Υץ SKK μԤޤ
 */

#include <config.h>
#include <stdio.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

/* ΥץǤϼ򤹤٤ƥɤ߹ߤޤ
   ͤϤ餫Ѱդȡ­ɲä
   ̤ꤷޤޤ꾮ˤ realloc()
   ȯޤ*/

#define STEP (256*1024)

/* ϼΥݥ󥿤ˤäƺ줿˽񤭹ޤޤ */
static unsigned char *dict;
static unsigned int dictsize;

/* ΥǥꥹȤǤ */
static unsigned int *index;

/* ȤǻѤݥ󥿤Ǥ */
static unsigned int *list;
static unsigned int n_line;

/* ɤ߹ߥץ */
static void
readdict()
{
  unsigned int bufsize, incsize;

  dictsize = 0;

  /* ѤΥȤ STEP ХȤѰդޤ */
  dict = malloc(STEP);
  bufsize = STEP;
  incsize = fread(dict+dictsize, 1, bufsize-dictsize, stdin);
  dictsize += incsize;

  /* ⤷դʤ STEP ХȤ䤷ɤ߹ߤ򷫤֤ޤ */
  while (dictsize == bufsize) {
    bufsize += STEP;
    dict = realloc(dict, bufsize);
    incsize = fread(dict+dictsize, 1, bufsize-dictsize, stdin);
    dictsize += incsize;
  }
}

/* ǥ */
static void
make_index()
{
  int i, n;

  n = 0;
  for (i = 0; i < dictsize; ++ i)
    if (dict[i] == '\n')
      ++ n;
  n_line = n;

  index = malloc((sizeof *index)*(n_line+1));
  n = 0;
  index[0] = 0;
  for (i = 0; i < dictsize; ++ i)
    if (dict[i] == '\n')
      index[++n] = i+1;

  list = malloc((sizeof *list)*n_line);
  for(i = 0; i < n_line; ++ i)
    list[i] = i;
}

/* Ȥʸӥ롼
   ʤΥȥǰ a ˤ٤ -1 ֤
   ʳλ 1 ֤ */
static int
hexcomp(a, b)
     unsigned char *a;
     unsigned char *b;
{
  while(*a == *b) {
    ++a; ++ b;
  }
  if (*a > *b) return 1;
  return -1;
}

/* ꤬ʤ뤫ɤȽǤ  0-ʤ 1- */
static int okuriari(p)
     unsigned char *p;
{
  if ((p[0] & 0x80) == 0) return 0;
  while(*p != ' ')
    if (*p == '\0') return 0; /* 򤬤ʤԤϰ۾Ȥư٤ */
    else            ++ p;
  if (('a' <= p[-1]) && (p[-1] <= 'z')) return 1;
  return 0;
}

/* Ȥӥ롼
    a ˤ٤ -1 ֤ʳλ 1 ֤ */
int skkcompar(a, b)
     int *a;
     int *b;
{
  unsigned char *ptra, *ptrb;
  ptra = dict+index[*a];
  ptrb = dict+index[*b];

  if (okuriari(ptra)) {
    if (okuriari(ptrb))
      /* 겾̾Τǡa b 򴹤Ӥ */
      return hexcomp(ptrb, ptra);
    else
      return -1;  /* a  */
  } else {
    if (okuriari(ptrb))
      return 1;  /* a  */
    else
      /* 겾̾ʤΤǡa b 򤽤ΤޤӤ̤֤ */
      return hexcomp(ptra, ptrb);
  }
}

/* ̤ */
static void printout()
{
  int i;
  unsigned char *ptr, *line;
  int okuriflag;

  puts(";; okuri-ari entries.");
  okuriflag = 1;
  for(i = 0; i < n_line; ++ i) {
    line = dict+index[list[i]];
    if (line[0] == ';') continue; /* ȹԤϽϤʤ */
    if (okuriflag) {
      if (!okuriari(line)) {
	puts(";; okuri-nasi entries.");
	okuriflag = 0;
      }
    }
    for (ptr = line; *ptr != '\n'; ++ ptr)
      putchar(*ptr);
    putchar('\n');
  }
}

/* ᥤץ ϻѤޤ */
int main(argc, argv)
     int argc;
     char **argv;
{
  readdict();
  make_index();
  qsort((char *)list, n_line, sizeof *list, skkcompar);
  printout();
  return 0;
}

/* end of skkdic-sort.c */

==> skktools-1.2.tar.gz: skktools-1.2/skkdic-sort.c <==
/* SKK JISYO TOOLS (SKK dictionary handling tools)
Copyright (C) 1994, 1996, 1999, 2000
      Hironobu Takahashi, Masahiko Sato, Kiyotaka Sakai

Author: Hironobu Takahashi, Masahiko Sato, Kiyotaka Sakai, Kenji Yabuuchi
Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
Version: $Id: skkdic-sort.c,v 1.5 2006/01/04 10:35:06 skk-cvs Exp $
Keywords: japanese
Last Modified: $Date: 2006/01/04 10:35:06 $

This file is part of Daredevil SKK.

SKK JISYO TOOLS are free software; you can redistribute them and/or modify
them under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

SKK JISYO TOOLS are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SKK; see the file COPYING.  If not, write to
the Free Software Foundation,  Inc., 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, USA. */

/* skkdic-sort.c
   Υץ SKK μԤޤ
 */

#include <config.h>
#include <stdio.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

/* ΥץǤϼ򤹤٤ƥɤ߹ߤޤ
   ͤϤ餫Ѱդȡ­ɲä
   ̤ꤷޤޤ꾮ˤ realloc()
   ȯޤ*/

#define STEP (256*1024)

/* ϼΥݥ󥿤ˤäƺ줿˽񤭹ޤޤ */
static unsigned char *dict;
static unsigned int dictsize;

/* ΥǥꥹȤǤ */
static unsigned int *index;

/* ȤǻѤݥ󥿤Ǥ */
static unsigned int *list;
static unsigned int n_line;

/* ɤ߹ߥץ */
static void
readdict()
{
  unsigned int bufsize, incsize;

  dictsize = 0;

  /* ѤΥȤ STEP ХȤѰդޤ */
  dict = malloc(STEP);
  bufsize = STEP;
  incsize = fread(dict+dictsize, 1, bufsize-dictsize, stdin);
  dictsize += incsize;

  /* ⤷դʤ STEP ХȤ䤷ɤ߹ߤ򷫤֤ޤ */
  while (dictsize == bufsize) {
    bufsize += STEP;
    dict = realloc(dict, bufsize);
    incsize = fread(dict+dictsize, 1, bufsize-dictsize, stdin);
    dictsize += incsize;
  }
}

/* ǥ */
static void
make_index()
{
  int i, n;

  n = 0;
  for (i = 0; i < dictsize; ++ i)
    if (dict[i] == '\n')
      ++ n;
  n_line = n;

  index = malloc((sizeof *index)*(n_line+1));
  n = 0;
  index[0] = 0;
  for (i = 0; i < dictsize; ++ i)
    if (dict[i] == '\n')
      index[++n] = i+1;

  list = malloc((sizeof *list)*n_line);
  for(i = 0; i < n_line; ++ i)
    list[i] = i;
}

/* Ȥʸӥ롼
   ʤΥȥǰ a ˤ٤ -1 ֤
   ʳλ 1 ֤ */
static int
hexcomp(a, b)
     unsigned char *a;
     unsigned char *b;
{
  while(*a == *b) {
    ++a; ++ b;
  }
  if (*a > *b) return 1;
  return -1;
}

/* ꤬ʤ뤫ɤȽǤ  0-ʤ 1- */
static int okuriari(p)
     unsigned char *p;
{
  if (((p[0] & 0x80) == 0) &&
      ((p[0] != '>' && p[0] != '#') || (p[1] & 0x80) == 0)) return 0;
  while(*p != ' ')
    if (*p == '\0') return 0; /* 򤬤ʤԤϰ۾Ȥư٤ */
    else            ++ p;
  if (('a' <= p[-1]) && (p[-1] <= 'z')) return 1;
  return 0;
}

/* Ȥӥ롼
    a ˤ٤ -1 ֤ʳλ 1 ֤ */
int skkcompar(a, b)
     int *a;
     int *b;
{
  unsigned char *ptra, *ptrb;
  ptra = dict+index[*a];
  ptrb = dict+index[*b];

  if (okuriari(ptra)) {
    if (okuriari(ptrb))
      /* 겾̾Τǡa b 򴹤Ӥ */
      return hexcomp(ptrb, ptra);
    else
      return -1;  /* a  */
  } else {
    if (okuriari(ptrb))
      return 1;  /* a  */
    else
      /* 겾̾ʤΤǡa b 򤽤ΤޤӤ̤֤ */
      return hexcomp(ptra, ptrb);
  }
}

/* ̤ */
static void printout()
{
  int i;
  unsigned char *ptr, *line;
  int okuriflag;

  puts(";; okuri-ari entries.");
  okuriflag = 1;
  for(i = 0; i < n_line; ++ i) {
    line = dict+index[list[i]];
    if (line[0] == ';') continue; /* ȹԤϽϤʤ */
    if (okuriflag) {
      if (!okuriari(line)) {
	puts(";; okuri-nasi entries.");
	okuriflag = 0;
      }
    }
    for (ptr = line; *ptr != '\n'; ++ ptr)
      putchar(*ptr);
    putchar('\n');
  }
}

/* ᥤץ ϻѤޤ */
int main(argc, argv)
     int argc;
     char **argv;
{
  readdict();
  make_index();
  qsort((char *)list, n_line, sizeof *list, skkcompar);
  printout();
  return 0;
}

/* end of skkdic-sort.c */
